/*===========================================================================
 * Project         : Mandrake Linux
 * Module          : rpmstats
 * File            : rpmstats.c
 * Version         : $Id: rpmstats.c 264547 2010-01-05 08:48:36Z pterjan $
 * Author          : Frederic Lepied
 * Created On      : Sat Feb 17 16:08:53 2001
 *===========================================================================*/

static const char rcs_id[] = "$Id: rpmstats.c 264547 2010-01-05 08:48:36Z pterjan $";
static const char compile_id[] = "$Compile: " __FILE__ " " __DATE__ " " __TIME__ " $";

/*
 **********************************************************************
 *
 * Included files
 *
 **********************************************************************
 */

#include <rpm/rpmlib.h>
#include <stdio.h>
#include <fcntl.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <sys/time.h>
#include <regex.h>
#include <string.h>

#include <rpm/rpmdb.h>
#include <rpm/rpmts.h>

int
main(int argc, char * argv[])
{
  rpmdbMatchIterator mi;
  rpmts ts;
  Header h;
  int data_size ;
  char ** file_list = NULL;
  struct timeval tv;
  regex_t rx;
  
  gettimeofday(&tv, NULL);

#ifdef DEBUG
  printf("%d\n", tv.tv_sec);
#endif
  
  rpmReadConfigFiles(NULL, NULL);

  ts = rpmtsCreate();
  rpmtsSetRootDir(ts, "");
  if (rpmtsOpenDB(ts, O_RDONLY)) {
    fprintf(stderr, "unable to open rpm database\n");
    exit(1);
  }

  mi = rpmtsInitIterator(ts, RPMDBI_PACKAGES, NULL, 0);
  
  if (regcomp(&rx, "/man/man\\|/usr/share/icons\\|/usr/lib/menu\\|/etc/menu-methods", 0)) {
    perror("regcomp");
    exit(1);
  }
  
  while ((h = rpmdbNextIterator(mi)) != NULL) {
    struct rpmtd_s val;
    char *nvr;
    int type, loop;
    struct stat buf;
    int max = 0;
    const char *file = NULL;
    const char *fname = NULL;
    
    headerGet(h, RPMTAG_NVR, &val, HEADERGET_EXT);
    nvr = (char *) rpmtdGetString(&val);
    if (nvr) {

      rpmfi fi;
      fi = rpmfiNew(NULL, h, RPMTAG_BASENAMES, 0);
      fi = rpmfiInit(fi, 0);
      while (rpmfiNext(fi) >= 0) {
	file = NULL;
        fname = rpmfiFN(fi);
	if (regexec(&rx, fname, 0, NULL, 0)) {
	  if (stat(fname, &buf) == 0) {
	    buf.st_atime = (buf.st_atime > buf.st_mtime) ? buf.st_atime : buf.st_mtime;
#ifdef DEBUG
	    fprintf(stderr, "%s %d\n", fname, buf.st_atime);
#endif
	    if (S_ISREG(buf.st_mode) && max < buf.st_atime) {
	      max = buf.st_atime;
	      file = fname;
	    }
	  } else {
	    perror(fname);
	  }
	} else {
#ifdef DEBUG
fprintf(stderr, "file %s match regex\n", fname);
#endif
	}
      }
      if (max != 0) {
	printf("%ld %s %s\n", (tv.tv_sec-max)/(3600*24), nvr, file);
      } else {
	printf("0 %s\n", nvr);
      }
      fi = rpmfiFree(fi);
      free(file_list);
    }
    rpmtdFreeData(&val);
  }

  rpmdbFreeIterator(mi);

  return 0;
}

/*
 * Local variables:
 * mode: c
 * compile-command: "gcc -Wall -g rpmstats.c -o rpmstats -lrpm -lrpmio -lpopt"
 * End:
 *
 * rpmstats.c ends here
 */
